from flask import render_template, abort, redirect, url_for, abort, flash, request, current_app, make_response, g
from flask_login import fresh_login_required, login_user, logout_user
from flask_sqlalchemy import get_debug_queries
from flask_paginate import Pagination, get_page_parameter

from . import admin
from .forms import ChargeForm, Addr2PoolForm, OrderForm

from .. import db
from ..email import send_email
from ..models import User, Transaction, Order, Server, Deployment, Service, Region, Address, Domain, contact_proxmaster
from ..decorators import admin_required, permission_required

import base64
import string
import random
from datetime import datetime, timedelta, date, time
import ipaddress

#@admin.before_app_request
#def before_request():
#    g.user = current_user
#    print('current_user: %s, g.user: %s, leaving bef_req' % (current_user, g.user))

@admin.after_app_request
def after_request(response):
    for query in get_debug_queries():
        if query.duration >= current_app.config['SLOW_DB_QUERY_TIME']:
            current_app.logger.warning('Slow query: %s\nParameters: %s\nDuration: %fs\nContext: %s\n' % (query.statement, query.parameters, query.duration, query.context))
    return response

@admin.route("/", methods=['GET'])
@fresh_login_required
@admin_required
def index():
    return redirect(url_for('admin.list_users'))

@admin.route("/listorders", methods=['GET', 'POST'])
@fresh_login_required
@admin_required
def list_orders():
    AllOrders = Order.query.filter_by(status='new').all()
    return render_template('admin/list_orders.html', orders=AllOrders)

@admin.route("/listdeployments", methods=['GET'])
@fresh_login_required
@admin_required
def list_deployments():
    AllDeploymentProtected = Deployment.query.filter_by(deleted=False).order_by(Deployment.daysleft.asc()).all()
    return render_template('admin/list_deployments.html', deployments=AllDeploymentProtected)

@admin.route("/listservices", methods=['GET'])
@fresh_login_required
@admin_required
def list_services():
    allservices = Service.query.filter_by(deleted=False).order_by(Service.daysleft.asc()).all()
    return render_template('admin/list_services.html', services=allservices)

@admin.route("/listdomains", methods=['GET'])
@fresh_login_required
@admin_required
def list_domains():
    alldomains = Domain.query.filter_by(deleted=False).order_by(Domain.daysleft.asc()).all()
    return render_template('admin/list_domains.html', domains=alldomains)

@admin.route("/listarchive", methods=['GET'])
@fresh_login_required
@admin_required
def list_archive():
    deployments = Deployment.query.filter_by(protected=False).order_by(Deployment.daysleft.asc()).all()
    services = Service.query.filter_by(deleted=True).all()
    domains = Domain.query.filter_by(deleted=True).all()
    return render_template('admin/list_archive.html', deployments=deployments, services=services, domains=domains)

@admin.route("/listusers", methods=['GET'])
@fresh_login_required
@admin_required
def list_users():
    search = False
    q = request.args.get('q')
    if q:
        search = True
    page = request.args.get(get_page_parameter(), type=int, default=1)
    per_page = 20
    sqlitems = User.query.filter_by(active=True).order_by(User.last_seen.desc()).all()
    pagination = Pagination(page=page, per_page=per_page, format_total=True, format_number=True, total=len(sqlitems), search=search, css_framework='foundation', record_name='users')
    return render_template('admin/list_users.html', users=sqlitems, page=page, per_page=per_page, pagination=pagination)

@admin.route("/charge/<int:user_pid>", methods=['GET', 'POST'])
@fresh_login_required
@admin_required
def charge(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()
    form = ChargeForm()
    if form.validate_on_submit():
        transaction = Transaction(user_id=int(cuser.pid), description='Account charged by staff', value=float(form.amount.data))
        db.session.add(transaction)
        db.session.commit()

        cuser.wallet += float(form.amount.data)
        db.session.add(cuser)
        db.session.commit()
        return redirect(url_for('admin.list_users'))
    return render_template('admin/charge.html', form=form, usr=cuser)

@admin.route("/listaddresses", methods=['GET'])
@fresh_login_required
@admin_required
def list_addresses():
    alladdresses = Address.query.all()
    alladdrlist = []
    for addr in alladdresses:
        alladdrlist.append(addr.ip)
    ipobjs = sorted(ipaddress.ip_address(addr) for addr in alladdrlist)
    ipnrml = []
    for ipobj in ipobjs:
        ipnrml.append(str(ipobj))
    alladdr = sorted(alladdresses, key=lambda o: ipnrml.index(o.ip))
    return render_template('admin/list_addresses.html', addresses=alladdr)

@admin.route("/addr2pool", methods=['GET', 'POST'])
@fresh_login_required
@admin_required
def addr2pool():
    alladdrlist = []
    alladdr = Address.query.all()
    for addr in alladdr:
        alladdrlist.append(str(addr.ip))
    #current_app.logger.info('Current IP pool: {}'.format(alladdrlist))
    form = Addr2PoolForm()
    if form.validate_on_submit():
        if form.ip.data in alladdrlist:
            flash('IP address {} is already in the pool!'.format(form.ip.data))
            return redirect(url_for('admin.addr2pool'))
        address = Address(ip=form.ip.data, rdns=form.rdns.data, region_id=form.region.data, enabled=True, reserved=form.reserved.data) 
        db.session.add(address)
        db.session.commit()
        flash('Address {} added to region {}'.format(form.ip.data, form.region.data))
        return redirect(url_for('admin.addr2pool'))
    return render_template('admin/addr2pool.html', form=form, alladdresses=alladdrlist) 

@admin.route("/listservers", methods=['GET'])
@fresh_login_required
@admin_required
def list_servers():
    allservers = Server.query.all()
    return render_template('admin/list_servers.html', servers=allservers)

@admin.route("/listtransactions", methods=['GET'])
@fresh_login_required
@admin_required
def list_transactions():
    alltransactions = Transaction.query.order_by(Transaction.date_created.desc()).all()
    return render_template('admin/list_transactions.html', transactions=alltransactions)

@admin.route("/transaction/<int:user_pid>", methods=['GET'])
@fresh_login_required
@admin_required
def transaction(user_pid=0):
    cuser = User.query.filter_by(pid=user_pid).first()
    transactions = cuser.inv_transactions.order_by(Transaction.date_created.desc()).limit(20)

    labelslist = ['today']
    translist = [cuser.wallet]
    prevvalue = cuser.wallet
    for tr in transactions:
        labelslist.insert(0, str(tr.date_created.strftime('%d.%m')))
        translist.insert(0, prevvalue - tr.value)
        prevvalue -= tr.value

    if len(labelslist) <= 1:
        labelslist.insert(0, 'before')
        translist.insert(0, 0)

    #current_app.logger.info('[{}] transactions: {} {} '.format(cuser.email, translist, labelslist))
    return render_template('uinvoice/transactions.html', transactions=transactions, translist=translist, labelslist=labelslist, cuser=cuser)

@admin.route("/dashboard/<int:user_pid>", methods=['GET'])
@fresh_login_required
@admin_required
def dashboard(user_pid=0):
    form = OrderForm()
    sys_regions = Region.query.all()

    cuser = User.query.filter_by(pid=user_pid).first()

    inv_deployments = cuser.inv_deployments.filter_by(deleted=False).order_by(Deployment.date_created.desc()).all()
    inv_deploycubeids = []
    inv_deployments_list = []
    for invcls in inv_deployments: 
        inv_deploycubeids.extend([invcls.machine_id]) 
        inv_deployments_list.extend([invcls.machine_alias]) 
 
    inv_services = cuser.inv_services.filter_by(deleted=False).order_by(Service.date_last_charge.asc()).all()
    inv_services_list = [] 
    for invcls in inv_services: 
        inv_services_list.extend([invcls.description]) 
                                                                                      
    inv_domains = cuser.inv_domains.filter_by(deleted=False).order_by(Domain.date_created.desc()).all()
    inv_domains_list = []
    for invcls in inv_domains:
        inv_domains_list.extend([invcls.fqdn])

    inv_addresses = cuser.inv_addresses.order_by(Address.ip.asc()).all()
    inv_addresses_list = []
    for invcls in inv_addresses:
        inv_addresses_list.extend([invcls.ip])

    sys_regions = Region.query.all()
    regions = {}
    for region in sys_regions:
        regions[region.pid] = region.description

    #extract rrd and status from the deployments
    rrd = {}
    statuses = {}
    for unit_id in inv_deploycubeids:
        rrd[unit_id] = {}
        data = { 'unit_id': int(unit_id),
                 'type': 'kvm' }
        query = contact_proxmaster(data, 'vmrrd')
        if query['status'] == 'UNREACHABLE':
            flash('Deploy #{} is unreachable.'.format(str(unit_id)))
            send_email(current_app.config['MAIL_USERNAME'], 'Deploy {} is unreachable'.format(unit_id), 
                                          'vmanager/email/adm_unreachable', user=cuser, unit_id=unit_id )
        else:
            graphs_list = ['net', 'cpu', 'mem', 'hdd']
            for graph in graphs_list:
                raw = query[graph]['image'].encode('raw_unicode_escape')
                rrd[unit_id][graph] = base64.b64encode(raw).decode()
            status = { unit_id : query['status'] }
            statuses.update(status)
    current_app.logger.info('[{}] Enabled deployments: {}, services: {}, domains: {},  addresses: {}'.format(cuser.email, inv_deployments_list, inv_services_list, inv_domains_list, inv_addresses_list))
    return render_template('panel/dashboard.html', form=form, sys_regions=sys_regions, inv_deployments=inv_deployments, inv_services=inv_services, inv_domains=inv_domains, inv_addresses=inv_addresses, rrd=rrd, status=statuses, regions=regions)

